class AudioPlayer {

	static get PANEL_HEIGHT() {return 100;};
	static get ID_LABEL_VOLUME() {return "AudioPlayerPanel-LabelVolume";};
	static get ID_LABEL_TRACK () {return "AudioPlayerPanel-LabelTrack" ;};
	static get ID_LABEL_TIME  () {return "AudioPlayerPanel-LabelTime"  ;};
	static get BUTTON_LOOP    () {return "AudioPlayerPanel-ButtonLoop"   ;};
	static get BUTTON_LOOPALL () {return "AudioPlayerPanel-ButtonLoopAll";};
	static get BUTTON_SELECT  () {return "AudioPlayerPanel-ButtonSelect" ;};
	static get BUTTON_PLAY    () {return "AudioPlayerPanel-ButtonPlay"   ;};
	static get CLASS_PLAYING  () {return "audioPlayerPanel-playing"  ;};
	static get CLASS_SELECTING() {return "audioPlayerPanel-selecting";};
	static get CLASS_ON       () {return "audioPlayerPanel-on"       ;};
	static get CLASS_TRACK_SELECTED () {return "audioPlayerTrackList-selected";};

	constructor(files, options) {
		options = this._setAttributes(options);
		this._files      = files;
		this._lskeyName  = options.localStoragePathName;
		this._interval   = options.interval;
		this._sequential = options.sequential;
		this._loop       = false;
		this._loopAll    = false;
		this._select     = false;
		this._selecting  = false;
		this._selection  = [];
		this._volume     = 100;
		this._trackNo    = 1;
		this._trackNoSet = false;
		this._timerId    = 0;

		this._audio = this._makeAudio();
		this._panel = this._makePanel(options);
		this._init();
	}

	set loop(val) {
		val = (val ? true : false);
		this._loop =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOP}`, val);
		if (val) {
			this._loopAll = false;
			this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOPALL}`, false);
		}
	}

	get loop() {
		return this._loop;
	}

	set loopAll(val) {
		val = (val ? true : false);
		this._loopAll =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOPALL}`, val);
		if (val) {
			this._loop = false;
			this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOP}`, false);
		}
	}

	get loopAll() {
		return this._loopAll;
	}

	set select(val) {
		val = (val ? true : false);
		this._select    =  val;
		this._selecting =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_SELECT}`, val);
		if (val) {
			this.pause();
			this._panel.classList.add(AudioPlayer.CLASS_SELECTING);
		} else {
			this._panel.classList.remove(AudioPlayer.CLASS_SELECTING);
			this._selection = [];
			qall(`.${AudioPlayer.CLASS_TRACK_SELECTED}`).forEach(item => {
				item.classList.remove(AudioPlayer.CLASS_TRACK_SELECTED);
			});
		}
	}

	get select() {
		return this._select;
	}

	appendTo(element) {
		if (!element) {
			return;
		}
		element.appendChild(this._audio);
		element.appendChild(this._panel);
	}

	appendTrackListTo(element) {
		if (!element) {
			return;
		}
		let list = document.createElement("ul");
		let len = this._files.length;
		for (let i = 0; i < len; i++) {
			let item = document.createElement("li");
			item.textContent     = this._files[i].title;
			item.dataset.trackNo = i + 1;
			item.tabIndex        = 0;
			AppUtil.doLikeButton(item, e => {
				if (this._selecting) {
					let trackNo = Number.parseInt(item.dataset.trackNo);
					let index   = this._selection.indexOf(trackNo);
					if (index < 0) {
						item.classList.add(AudioPlayer.CLASS_TRACK_SELECTED);
						this._selection.push(trackNo);
					} else {
						item.classList.remove(AudioPlayer.CLASS_TRACK_SELECTED);
						this._selection = this._selection.filter((el, i) => i != index);
					}
				} else if (!this._select || item.classList.contains(AudioPlayer.CLASS_TRACK_SELECTED)) {
					this.setTrackNo(item.dataset.trackNo);
					this.play();
				}
			}, true);
			list.appendChild(item);
		}
		element.appendChild(list);
	}

	setTrackNo(trackNo) {
		if (trackNo == null || isNaN(trackNo)) {
			trackNo = 1;
		} else {
			trackNo = Number.parseInt(trackNo);
			if (trackNo < 1) {
				trackNo = this._files.length;
			} else if (trackNo > this._files.length) {
				trackNo = 1;
			}
		}
		this._trackNoSet = true;
		this._trackNo = trackNo;
		this._setTextContent(`#${AudioPlayer.ID_LABEL_TRACK} > *:last-child`, trackNo);
		this._clearTime();
		return this._trackNo;
	}

	play() {
		if (this._selecting) {
			this._selecting = false;
			this._panel.classList.remove(AudioPlayer.CLASS_SELECTING);
			if (this._selection.length > 0) {
				this.setTrackNo(this._selection[0]);
			} else {
				this.select = false;
			}
		}
		if (this._trackNoSet) {
			this._audio.pause();
			this._audio.src = this._files[this._trackNo - 1].path;
			this._audio.load();
			this._trackNoSet = false;
		}
		this._setPlayingTrack(true);
		this._audio.play();
		this._displayTime();
	}

	pause() {
		this._audio.pause();
		this._stopTimer();
		this._setPlayingTrack(false);
	}

	next() {
		let trackNo = 0;
		if (this._select) {
			this.setTrackNo(this._searchTrackNoInSelection(true));
		} else {
			this.setTrackNo(this._trackNo + 1);
		}
		if (!this._audio.paused) {
			this.play();
		}
	}

	prev() {
		let paused = this._audio.paused;
		this._audio.pause();
		if (this._audio.currentTime > 1) {
			this._resetCurrentTime();
			this._audio.currentTime = 0;
		} else if (this._select) {
			this.setTrackNo(this._searchTrackNoInSelection(false));
		} else {
			this.setTrackNo(this._trackNo - 1);
		}
		if (!paused) {
			this.play();
		}
	}

	turnUpVolume() {
		return this.setVolume(this._volume + 5);
	}

	turnDownVolume() {
		return this.setVolume(this._volume - 5);
	}

	setVolume(volume) {
		if (volume == null || isNaN(volume)) {
			volume = 0;
		} else {
			volume = Number.parseInt(volume);
			if (volume < 0) {
				volume = 0;
			} else if (volume > 100) {
				volume = 100;
			}
		}
		if (this._lskeyName) {
			this._setItemToLocalStorage(`volume@${this._lskeyName}`, volume);
		}
		this._audio.volume = volume * 0.01;
		this._volume = volume;
		this._setTextContent(`#${AudioPlayer.ID_LABEL_VOLUME} > *:last-child`, volume);
		return this._volume;
	}

	_init() {
		if (this._lskeyName) {
			let vol = this._getItemFromLocalStorage(`volume@${this._lskeyName}`);
			if (vol) {
				this.setVolume(vol);
			}
		}
		this.setTrackNo(1);
		this._audio.src = this._files[0].path;
		this._audio.load();
		this._audio.addEventListener("ended", e => {
			this._handleEndedEvent();
		});
	}

	_handleEndedEvent() {
		this._stopTimer();
		if (this._loop) {
			this._resetCurrentTime();
			if (this._playingTrack()) {
				this.play();
			}
		} else if (this._sequential || this._loopAll) {
			let interval = this._files[this._trackNo - 1].interval;
			if (interval == null || isNaN(interval)) {
				interval = this._interval;
			}

			if (this._loopAll || this._trackNo < this._files.length) {
				this.next();
			} else {
				this._setPlayingTrack(false);
				return;
			}
			if (interval == 0) {
				this.play();
			} else {
				setTimeout(() => {
					if (this._playingTrack()) {
						this.play();
					}
				}, interval);
			}
		} else {
			this._resetCurrentTime();
			this._setPlayingTrack(false);
		}
	}

	_setPlayingTrack(val) {
		val = (val ? true : false);
		if (val) {
			this._panel.classList.add(AudioPlayer.CLASS_PLAYING);
		} else {
			this._panel.classList.remove(AudioPlayer.CLASS_PLAYING);
		}
	}

	_playingTrack() {
		return this._panel.classList.contains(AudioPlayer.CLASS_PLAYING);
	}

	_displayTime() {
		this._stopTimer();
		setTimeout(() => {
			this._displayTotalTime();
			this._displayCurrentTime();
			setTimeout(() => {
				this._timerId = setInterval(() => {
					this._displayCurrentTime();
					// this._displayTotalTime();
				}, 1000);
			}, this._calcDelayTime());
		}, 200);
	}

	_calcDelayTime() {
		let cur = this._audio.currentTime;
		let d = Math.floor((Math.ceil(cur) - cur) * 1000);
		return d;
	}

	_stopTimer() {
		clearInterval(this._timerId);
	}

	_resetCurrentTime() {
		this._setTextCurrentTime("0:00");
	}

	_clearTime() {
		this._setTextCurrentTime("");
		this._setTextTotalTime("");
	}

	_displayCurrentTime() {
		let cur = this._formatTime(Math.round(this._audio.currentTime));
		this._setTextCurrentTime(cur);
	}

	_displayTotalTime() {
		let total = this._formatTime(Math.ceil (this._audio.duration));
		this._setTextTotalTime(total);
	}

	_setTextCurrentTime(text) {
		this._setTextContent(`#${AudioPlayer.ID_LABEL_TIME} > *:nth-child(2)`, text);
	}

	_setTextTotalTime(text) {
		this._setTextContent(`#${AudioPlayer.ID_LABEL_TIME} > *:nth-child(4)`, text);
	}

	_formatTime(sec) {
		if (sec >= 5999) {
			return "99:59"
		}
		let m = (Math.floor(sec / 60)).toString();
		let s = (sec % 60).toString();
		if (s.length < 2) {
			s = "0" + s;
		}
		return m + ":" + s;
	}

	_setTextContent(selector, text) {
		this._panel.querySelector(selector).textContent = text;
	}

	_setFunctionOn(selector, on) {
		let element = this._panel.querySelector(selector);
		if (!element) {
			return;
		}
		if (on) {
			element.classList.add(AudioPlayer.CLASS_ON);
		} else {
			element.classList.remove(AudioPlayer.CLASS_ON);
		}
	}

	_searchTrackNoInSelection(forward) {
		let index = this._selection.indexOf(this._trackNo) + (forward ? 1 : -1);
		if (index < 0) {
			index = this._selection.length - 1;
		} else if (index >= this._selection.length) {
			index = 0;
		}
		return this._selection[index];
	}

	_setItemToLocalStorage(key, val) {
		try {
			localStorage.setItem(key, val);
		} catch (e) {
		}
	}

	_getItemFromLocalStorage(key) {
		let ret = null;
		try {
			ret = localStorage.getItem(key);
		} catch (e) {
		}
		return ret;
	}

	_makeAudio() {
		let audio = document.createElement("audio");
		if (this._files && this._files.length) {
			audio.src = this._files[0].path;
		}
		audio.style = "display: none";
		return audio;
	}

	_makePanel(options) {
		let panel = this._makeSvgElement("svg");
		panel.appendChild(this._makeStyle(options.color, options.backgroundColor, options.hoveringColor, options));
		panel.appendChild(this._makeDefs());
		this._addParts(panel, options);
		return panel;
	}

	_addParts(svg, options) {
		let h = AudioPlayer.PANEL_HEIGHT;
		let w = 0;
		if (options.functions) {
			this._makeFuntionsPanel(svg, w, options.backgroundColor);
			w += 100;
		}
		if (options.controls) {
			this._makeControlsPanel(svg, w, options.backgroundColor);
			w += 200;
		}
		if (options.volume) {
			this._makeVolumePanel(svg, w, options.backgroundColor);
			w += 50;
		}

		svg.setAttribute("id", "AudioPlayerPanel");
		svg.setAttribute("width"  , w);
		svg.setAttribute("height" , h);
		svg.setAttribute("viewBox", `0 0 ${w} ${h}`);
		svg.setAttribute("version", "1.1");
	}

	_makeStyle(color, backgroundColor, hoveringColor, options) {
		let style = this._makeSvgElement("style");
		style.textContent = `
		/* <![CDATA[ */
		#AudioPlayerPanel {
			color: ${color};
			letter-spacing: normal;
		}
		text {
			font-family: "Arial Black", sans-serif;
			pointer-events: none;
		}
		#AudioPlayerPanel.${AudioPlayer.CLASS_PLAYING} g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(2) {
			display:none;
		}
		#AudioPlayerPanel:not(.${AudioPlayer.CLASS_PLAYING}) > g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(3) {
			display:none;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON} use {
			fill: currentColor;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON} text {
			fill: ${backgroundColor};
		}
		#AudioPlayerPanel > g:hover {
			cursor: pointer;
		}
		#AudioPlayerPanel > g:hover use:first-child {
			fill: ${hoveringColor};
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON}:hover use {
			fill: currentColor;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON}:hover text {
			fill: ${hoveringColor};
		}

		#AudioPlayerPanel.${AudioPlayer.CLASS_SELECTING} > g#${AudioPlayer.BUTTON_PLAY} > * {
			animation-iteration-count: infinite;
			animation-duration: 1600ms;
			animation-timing-function: linear;
			animation-direction: alternate;
		}
		#AudioPlayerPanel.${AudioPlayer.CLASS_SELECTING} > g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(1) {
			animation-name: ready2play1;
		}
		#AudioPlayerPanel.${AudioPlayer.CLASS_SELECTING} > g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(2) {
			animation-name: ready2play2;
		}
		@keyframes ready2play1 {
			  0% {fill: transparent     ; filter: drop-shadow(0 0 4px transparent     );}
			 75% {fill: transparent     ; filter: drop-shadow(0 0 4px transparent     );}
			 95% {fill: ${hoveringColor}; filter: drop-shadow(0 0 4px ${hoveringColor});}
			100% {fill: ${hoveringColor}; filter: drop-shadow(0 0 4px ${hoveringColor});}
		}
		@keyframes ready2play2 {
			  0% {opacity: 1.0;}
			 75% {opacity: 1.0;}
			 95% {opacity: 0.6;}
			100% {opacity: 0.6;}
		}`;

		if (!options.time) {
			style.textContent += `#${AudioPlayer.ID_LABEL_TIME} {display: none;}`;
		}
		style.textContent += `/* ]]> */`;

		return style;
	}

	_makeDefs() {
		let defs = this._makeSvgElement("defs");
		defs.appendChild(this._makePath("AudioPlayerPanel-FunctionButton"   ,
		"M -28 -12 H  28 A  12 12 0 0 1 28  12 H -28 A 12 12 0 0 1 -28 -12 Z"));

		defs.appendChild(this._makePath("AudioPlayerPanel-VolumePlusButton" ,
		"M -12 0 V -18 A 12 12 0 0 1 12 -18 V 0"));

		defs.appendChild(this._makePath("AudioPlayerPanel-VolumeMinusButton",
		"M 12 0 V 18 A 12 12 0 0 1 -12 18 V 0"));

		defs.appendChild(this._makeCircle("AudioPlayerPanel-ControlButton", 24));

		let pause = this._makeGroupInDefs("AudioPlayerPanel-SymbolPause", "scale(0.3, 0.3)");
		pause.appendChild(this._makeRect(-33, -50, 24, 100));
		pause.appendChild(this._makeRect(  9, -50, 24, 100));
		defs.appendChild(pause);

		let play  = this._makeGroupInDefs("AudioPlayerPanel-SymbolPlay" , "scale(0.3, 0.3)");
		play.appendChild(this._makePolyline("-14.6 -50, -14.6 50, 35.4 0, -14.6 -50"));
		defs.appendChild(play);

		let prev  = this._makeGroupInDefs("AudioPlayerPanel-SymbolPrev" , "scale(-0.3, 0.3)");
		prev.appendChild(this._makeRect(21, -50, 12, 100));
		prev.appendChild(this._makePolyline("-33 -50,-33 50,17 0, -33 -50"));
		defs.appendChild(prev);

		let next  = this._makeGroupInDefs("AudioPlayerPanel-SymbolNext" , "scale(0.3, 0.3)" );
		next.appendChild(this._makeRect(21, -50, 12, 100));
		next.appendChild(this._makePolyline("-33 -50,-33 50,17 0, -33 -50"));
		defs.appendChild(next);

		return defs;
	}

	_makePath(id, d) {
		let p = this._makeSvgElement("path");
		p.setAttribute("id", id);
		p.setAttribute("d" , d );
		p.setAttribute("stroke-width", 2);
		p.setAttribute("stroke", "currentColor");
		return p;
	}

	_makeCircle(id, r) {
		let c = this._makeSvgElement("circle");
		c.setAttribute("id", id);
		c.setAttribute("r" , r );
		c.setAttribute("stroke-width", 2);
		c.setAttribute("stroke", "currentColor");
		return c;
	}

	_makeGroupInDefs(id, transform) {
		let g = this._makeSvgElement("g");
		g.setAttribute("id", id);
		g.setAttribute("stroke-width"  , 0);
		g.setAttribute("pointer-events", "none");
		g.setAttribute("transform", transform);
		return g;
	}


	_makeRect(x, y, w, h) {
		let r = this._makeSvgElement("rect");
		r.setAttribute("x"     , x);
		r.setAttribute("y"     , y);
		r.setAttribute("width" , w);
		r.setAttribute("height", h);
		return r;
	}

	_makePolyline(points) {
		let p = this._makeSvgElement("polyline");
		p.setAttribute("points", points);
		return p;
	}

	_makeFuntionsPanel(svg, xOffset, bgColor) {
		let x = 50 + xOffset;
		let loop = this._makeSvgElement("g");
		loop.setAttribute("id", AudioPlayer.BUTTON_LOOP);
		loop.appendChild(this._makeUse (x, 18, "AudioPlayerPanel-FunctionButton", bgColor));
		loop.appendChild(this._makeText(x, 22, "middle", 12, "LOOP"));
		loop.addEventListener("click", e => {this.loop = !this.loop;});
		svg.appendChild(loop);

		let all = this._makeSvgElement("g");
		all.setAttribute("id", AudioPlayer.BUTTON_LOOPALL);
		all.appendChild(this._makeUse (x, 50, "AudioPlayerPanel-FunctionButton", bgColor));
		all.appendChild(this._makeText(x, 54, "middle", 12, "LOOP ALL"));
		all.addEventListener("click", e => {this.loopAll = !this.loopAll;});
		svg.appendChild(all);

		let select = this._makeSvgElement("g");
		select.setAttribute("id", AudioPlayer.BUTTON_SELECT);
		select.appendChild(this._makeUse (x, 82, "AudioPlayerPanel-FunctionButton", bgColor));
		select.appendChild(this._makeText(x, 86, "middle", 12, "SELECT"));
		select.addEventListener("click", e => {this.select = !this.select;});
		svg.appendChild(select);
	}

	_makeControlsPanel(svg, xOffset, bgColor) {
		let track = this._makeSvgElement("g");
		track.setAttribute("id", AudioPlayer.ID_LABEL_TRACK);
		track.appendChild(this._makeText(10 + xOffset, 14, "start", 12, "TRACK"));
		track.appendChild(this._makeText(57 + xOffset, 33, "end"  , 17, "0"));
		svg.appendChild(track);

		
		let time = this._makeSvgElement("g");
		time.setAttribute("id", AudioPlayer.ID_LABEL_TIME);
		time.appendChild(this._makeText( 75 + xOffset, 14, "start" , 12, "TIME"));
		time.appendChild(this._makeText(126 + xOffset, 33, "end"   , 17, ""));
		time.appendChild(this._makeText(133 + xOffset, 33, "middle", 17, "/"));
		time.appendChild(this._makeText(190 + xOffset, 33, "end"   , 17, ""));
		svg.appendChild(time);

		let prev = this._makeSvgElement("g");
		prev.appendChild(this._makeUse ( 35 + xOffset, 70, "AudioPlayerPanel-ControlButton", bgColor));
		prev.appendChild(this._makeUse ( 35 + xOffset, 70, "AudioPlayerPanel-SymbolPrev" , "currentColor" ));
		prev.addEventListener("click", e => {this.prev();});
		svg.appendChild(prev);

		let next = this._makeSvgElement("g");
		next.appendChild(this._makeUse (165 + xOffset, 70, "AudioPlayerPanel-ControlButton", bgColor));
		next.appendChild(this._makeUse (165 + xOffset, 70, "AudioPlayerPanel-SymbolNext" , "currentColor" ));
		next.addEventListener("click", e => {this.next();});
		svg.appendChild(next);

		let play = this._makeSvgElement("g");
		play.setAttribute("id", AudioPlayer.BUTTON_PLAY);
		play.appendChild(this._makeUse (100 + xOffset, 70, "AudioPlayerPanel-ControlButton", bgColor));
		play.appendChild(this._makeUse (100 + xOffset, 70, "AudioPlayerPanel-SymbolPlay" , "currentColor" ));
		play.appendChild(this._makeUse (100 + xOffset, 70, "AudioPlayerPanel-SymbolPause", "currentColor" ));
		play.addEventListener("click", e => {
			if (this._playingTrack()) {
				this.pause();
			} else {
				this.play();
			}
		});
		svg.appendChild(play);
	}

	_makeVolumePanel(svg, xOffset, bgColor) {
		let vol = this._makeSvgElement("g");
		vol.setAttribute("id", AudioPlayer.ID_LABEL_VOLUME);
		vol.appendChild(this._makeText(38 + xOffset, 14, "end", 12, "VOL"));
		vol.appendChild(this._makeText(37 + xOffset, 95, "end", 12, "100"));
		svg.appendChild(vol);

		let plus = this._makeSvgElement("g");
		plus.appendChild(this._makeUse (25 + xOffset, 50, "AudioPlayerPanel-VolumePlusButton", bgColor));
		plus.appendChild(this._makeText(25 + xOffset, 38, "middle", 20, "+"));
		plus.addEventListener("click", e => {this.turnUpVolume();});
		svg.appendChild(plus);

		let minus = this._makeSvgElement("g");
		minus.appendChild(this._makeUse (25 + xOffset, 50, "AudioPlayerPanel-VolumeMinusButton", bgColor));
		minus.appendChild(this._makeText(25 + xOffset, 74, "middle", 20, "-"));
		minus.addEventListener("click", e => {this.turnDownVolume();});
		svg.appendChild(minus);
	}

	_makeUse(x, y, linkedId, backgroundColor) {
		let u = this._makeSvgElement("use");
		u.setAttribute("x", x);
		u.setAttribute("y", y);
		u.setAttribute("fill"      , backgroundColor);
		u.setAttributeNS("http://www.w3.org/1999/xlink", "href", `#${linkedId}`);
		return u;
	}

	_makeText(x, y, textAnchor, fontSize, text) {
		let t = this._makeSvgElement("text");
		t.setAttribute("x", x);
		t.setAttribute("y", y);
		t.setAttribute("text-anchor", textAnchor    );
		t.setAttribute("font-size"  , fontSize      );
		t.setAttribute("fill"       , "currentColor");
		t.textContent = text;
		return t;
	}

	_makeSvgElement(name) {
		return document.createElementNS("http://www.w3.org/2000/svg", name);
	}

	_setAttributes(options) {
		let attr = this._defaultAttributes();
		if (!options) {
			return attr;
		}

		if (options.color) {
			attr.color = options.color;
		}
		if (options.backgroundColor) {
			attr.backgroundColor = options.backgroundColor;
		}

		attr.sequential = (options.sequential ? true : false);
		if (options.interval != null && !isNaN(options.interval)) {
			attr.interval = Number.parseInt(options.interval);
		}

		if (options.localStoragePathName) {
			attr.localStoragePathName = options.localStoragePathName;
		}

		if ("functions" in options) {
			attr.functions = (options.functions ? true : false);
		}
		if ("controls" in options) {
			attr.controls = (options.controls ? true : false);
		}
		if ("time" in options) {
			attr.time = (options.time ? true : false);
		}
		if ("volume" in options) {
			attr.volume = (options.volume ? true : false);
		}

		return attr;
	}

	_defaultAttributes() {
		return {
			color          : "green",
			backgroundColor: "white",
			hoveringColor  : "yellow",
			sequential     : false,
			interval       : 1000,
			localStoragePathName: null,
			functions: true,
			controls : true,
			time     : true,
			volume   : true
		};
	}
}
